/*========================================================================
 * Copyright (C) GemTalk Systems 1986-2013.  All Rights Reserved..
 *
 * Name - fileopentest.c 
 * Description: 
 *   example program to test how many files can be simultaneously 
 *   opened by a process .  
 *
 *   For SunOs only
 *
 * $Id: fileopentest.c,v 1.9 2008-01-09 19:03:03 stever Exp $
 *
 *========================================================================
 */
#if defined(AIX) && defined(__GNUC__)
/* for gcc 2.95.3 */
#define _ALL_SOURCE 1
#define _32KERMODE 1
#include <sys/types.h>
#endif

#if defined(__hpux)
#if !defined(_HPUX_SOURCE)
#define _HPUX_SOURCE
#endif /* !defined(_HPUX_SOURCE) */
#include <sys/stdsyms.h> /* Pick up the other related #defs */
#endif

#include <errno.h>
#include <stdio.h>

#include <sys/time.h>
#if !defined(_SEQUENT_)
#include <sys/resource.h>
#endif

#if defined(hpux) || defined(__hpux) || defined(sparcSolaris) \
  || (defined(_nst) && defined(_ATT4) && defined(i386)) || defined(sinix)
#include <unistd.h>
#define getdtablesize() sysconf(_SC_OPEN_MAX)
#endif

#if !defined(RLIMIT_NOFILE)
#define RLIMIT_NOFILE 6
#endif

#if !defined(RLIM_INFINITY)
#define RLIM_INFINITY 0x7fffffff
#endif

void printLimits()
{   
#if !defined(_SEQUENT_)
    struct rlimit limStruct;
    long rlimResult;

    rlimResult = getrlimit(RLIMIT_NOFILE, &limStruct);
    if (rlimResult != 0) {
      printf("getrlimit returned %ld\n", rlimResult);
      printf("errno = %ld\n", errno);
      perror("host error text = ");
      }
    else
      printf("getrlimit returns:  rlim_cur = %ld, rlim_max = %ld\n", 
                       limStruct.rlim_cur, limStruct.rlim_max);
#endif /* _SEQUENT_ */
    printf("getdtablesize returns  %ld\n", getdtablesize() );
}

int main (int argc, const char *argv)
{
    long i, fileLimit ;
    char fileName[100];
    long myPid;
#if !defined(_SEQUENT_)
    struct rlimit limStruct;
    long rlimResult;
#endif

    myPid = getpid();

    printLimits();
#if !defined(_SEQUENT_)
    limStruct.rlim_cur = RLIM_INFINITY;
    limStruct.rlim_max = RLIM_INFINITY;
    printf("Now calling setrlimit to request max possible file descriptors.\n");
    rlimResult = setrlimit(RLIMIT_NOFILE, &limStruct);
    if (rlimResult != 0) {
      printf("setrlimit returned %ld\n", rlimResult);
      printf("errno = %ld\n", errno);
      perror("host error text = ");
      }
    printLimits();
    fileLimit = limStruct.rlim_cur - 5; 
#else
    fileLimit = getdtablesize() - 5;
#endif
    i = 1;
    while (i < fileLimit) {
      sprintf(fileName, "/tmp/fileopen%ld_%ld.out\0",myPid, i);  
      if ( fopen(fileName, "w" ) == NULL ) {
        printf("Failed on file %d \n", i);
        printf("filename = %s\n", fileName); 
        printf("errno = %ld\n", errno);
        perror("host error text = ");
        if (i == 1)
          printf("You may need to  mkdir ./temp \n");
	return;
        }
      if ((i % 25) == 0) 
        printf("successfully opened file %d \n", i);
      i = i + 1;
      }
    return 0; // lint
  }

